/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#include "psq.h"

int PSQTable[NB_PHASE][NB_PIECE][NB_SQUARE];
bool PSQInitialized = false;

void init_psq()
{
	if (PSQInitialized) return;
	else PSQInitialized = true;

	for (int phase = Opening; phase <= EndGame; phase++)
		for (int piece = Pawn; piece <= King; piece++)
			for (int sq = A1; sq <= H8; sq++)
				PSQTable[phase][piece][sq] = Material[phase][piece] + PSQBonus[phase][piece][sq];
}

static int calc_psq(const U64 b[NB_PIECE], int phase, int color)
{
	assert(PSQInitialized && phase_ok(phase));
	int result = 0;

	for (int piece = Pawn; piece <= King; piece++) {
		U64 sqs = b[piece];
		while (sqs) {
			int sq = next_bit(&sqs);
			result += get_psqtab(color, phase, piece, sq);
		}
	}

	return result;
}

bool psq_ok(const Board *B)
{
	for (int phase = Opening; phase <= EndGame; phase++)
		for (int color = White; color <= Black; color++)
			if (B->st->psq[color][phase] != calc_psq(B->b[color], phase, color))
				return false;
	return true;
}